package com.huawei.cff;

import java.io.IOException;

import org.apache.http.HttpEntity;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPut;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import java.util.Base64;

import net.sf.json.JSONArray;
import net.sf.json.JSONObject;

import com.huawei.services.runtime.Context;
import com.huawei.services.runtime.RuntimeLogger;

public class TableTester {
    private CloseableHttpClient httpclient;
    private RuntimeLogger LOG;
    private String strToken;
    private final String tableName = "cff_cloud_table";
    private final String columnFamilyName = "f1";
    private final String seqKey = "sequence_number";
    private final String dataKey = "data";
    private final String partKey = "partition_key";
    private String hostName = "100.125.1.131:8080";
    private String proID = "644d695dbfc14eb08f1125b4d456c548";
    private String clusID = "9fa86af7-81bc-4e21-975f-9e9fc0899fdc";
    
    private String rowKey = "";

    public String MyHandler(String param, Context context)
            throws ClientProtocolException, IOException {
        LOG = context.getLogger();
        httpclient = HttpClients.createDefault();
        strToken = context.getToken();
        if (strToken != null && !strToken.isEmpty()) {
            LOG.log("token length : " + strToken.length());
        } else {
            LOG.log("token is empty!");
        }

        String uProID = context.getUserData("ProjectID");
        String uClusID = context.getUserData("ClusterID");
        String uHostName = context.getUserData("HostName");

        if (uProID != null && !uProID.isEmpty()) {
            proID = uProID;
            LOG.log("project_id : " + uProID);
        }

        if (uClusID != null && !uClusID.isEmpty()) {
            clusID = uClusID;
            LOG.log("cluster_id : " + uClusID);
        }

        if (uHostName != null && !uHostName.isEmpty()) {
            hostName = uHostName;
            LOG.log("host_name : " + uHostName);
        }

        try {
            GetAllTable();
            CreateTable();
            SaveDISData(param);
            return "OK";
        } finally {
            httpclient.close();
        }
    }

    public void GetAllTable() throws ClientProtocolException, IOException {
        String strUrl =
            "http://" + hostName + "/v1.0/" + proID + "/clusters/" + clusID
                    + "/hbase/";
        LOG.log("URL: " + strUrl);
        HttpGet httpGet = new HttpGet(strUrl);
        httpGet.addHeader("X-Auth-Token", strToken);
        httpGet.addHeader("Content-Type", "application/json");
        httpGet.addHeader("Accept", "application/json");
        CloseableHttpResponse response = httpclient.execute(httpGet);
        try {
            LOG.log(response.getStatusLine().toString());
            HttpEntity entity = response.getEntity();

            if (entity != null) {
                LOG.log(EntityUtils.toString(entity));
            }
        } finally {
            response.close();
        }
    }

    public void CreateTable() throws ClientProtocolException, IOException {
        String strJson =
            "{\"name\": \"" + tableName + "\",\"ColumnSchema\": [{\"name\": \""
                    + columnFamilyName + "\"}],\"IS_META\": \"false\"}";
        String strUrl =
            "http://" + hostName + "/v1.0/" + proID + "/clusters/" + clusID
                    + "/hbase/" + tableName + "/schema";

        HttpPut httpPut = new HttpPut(strUrl);
        httpPut.addHeader("X-Auth-Token", strToken);
        httpPut.addHeader("Content-Type", "application/json");
        httpPut.addHeader("Accept", "application/json");

        StringEntity se = new StringEntity(strJson);
        httpPut.setEntity(se);

        CloseableHttpResponse response = httpclient.execute(httpPut);
        try {
            LOG.log(response.getStatusLine().toString());
            HttpEntity entity = response.getEntity();

            if (entity != null) {
                LOG.log(EntityUtils.toString(entity));
            }
        } finally {
            response.close();
        }
    }

    public void InsertRowData(String strPart, String strSeq, String strData)
            throws ClientProtocolException, IOException {
        String strRowKey = "row" + strSeq;
        rowKey = strRowKey;
        String strRowBase64 =
            Base64.getEncoder().encodeToString(strRowKey.getBytes("utf-8"));

        String strseqKey = columnFamilyName + ":" + seqKey;
        String strSeqKeyBase64 =
            Base64.getEncoder().encodeToString(strseqKey.getBytes("utf-8"));

        String strPartKey = columnFamilyName + ":" + partKey;
        String partKeyBase64 =
            Base64.getEncoder().encodeToString(strPartKey.getBytes("utf-8"));

        String strDataKey = columnFamilyName + ":" + dataKey;
        String dataKeyBase64 =
            Base64.getEncoder().encodeToString(strDataKey.getBytes("utf-8"));

        String seqValueBase64 =
            Base64.getEncoder().encodeToString(strSeq.getBytes("utf-8"));
        String partValueBase64 =
            Base64.getEncoder().encodeToString(strPart.getBytes("utf-8"));
        String dataValueBase64 = strData;

        String strJson =
            "{\"Row\": [{\"key\": \"" + strRowBase64 + "\",\"Cell\": [{\"column\": \"" +strSeqKeyBase64 + "\",\"$\": \"" + seqValueBase64 + "\"},{\"column\": \"" + partKeyBase64 + "\",\"$\": \""+ partValueBase64 + "\"},{\"column\": \""+ dataKeyBase64 + "\",\"$\": \"" + dataValueBase64 +"\"}]}]}";
        String strUrl =
            "http://" + hostName + "/v1.0/" + proID + "/clusters/" + clusID
                    + "/hbase/" + tableName + "/" + strRowKey;

        LOG.log("Insert data : " + strJson);
        LOG.log("Insert url : " + strUrl);
        HttpPut httpPut = new HttpPut(strUrl);
        httpPut.addHeader("X-Auth-Token", strToken);
        httpPut.addHeader("Content-Type", "application/json");
        httpPut.addHeader("Accept", "application/json");

        StringEntity se = new StringEntity(strJson);
        httpPut.setEntity(se);

        CloseableHttpResponse response = httpclient.execute(httpPut);
        try {
            LOG.log(response.getStatusLine().toString());
            HttpEntity entity = response.getEntity();

            if (entity != null) {
                LOG.log(EntityUtils.toString(entity));
            }
        } finally {
            response.close();
        }
    }

    public void SaveDISData(String strInput) throws IOException {
        JSONObject jsonObject =
            JSONObject.fromObject(strInput).getJSONObject("Message");

        JSONArray jsonArray = jsonObject.getJSONArray("records");

        int size = jsonArray.size();
        for (int i = 0; i < size; i++) {
            JSONObject record = jsonArray.getJSONObject(i);

            String strSeq = record.getString(seqKey);
            String strData = record.getString(dataKey);
            String strPartKey = record.getString(partKey);

            LOG.log("partition_key : " + strPartKey + " sequence_number : "
                    + strSeq + " data : " + strData);

            InsertRowData(strPartKey, strSeq, strData);
            GetARowData();
        }
    }
    
    
    public void GetARowData() throws ClientProtocolException, IOException {
        String strUrl =
                "http://" + hostName + "/v1.0/" + proID + "/clusters/" + clusID
                        + "/hbase/" + tableName + "/" + rowKey;
            LOG.log("URL: " + strUrl);
            HttpGet httpGet = new HttpGet(strUrl);
            httpGet.addHeader("X-Auth-Token", strToken);
            httpGet.addHeader("Content-Type", "application/json");
            httpGet.addHeader("Accept", "application/json");
            CloseableHttpResponse response = httpclient.execute(httpGet);
            try {
                LOG.log(response.getStatusLine().toString());
                HttpEntity entity = response.getEntity();

                if (entity != null) {
                    LOG.log(EntityUtils.toString(entity));
                }
            } finally {
                response.close();
            }
    }
}
